@@ -1,6 +1,6 @@
 #
-# Copyright CEA/DAM/DIF (2009-2014)
-#  Contributor: Stephane THIELL <stephane.thiell@cea.fr>
+# Copyright CEA/DAM/DIF (2009-2016)
+#  Contributor: Stephane THIELL <sthiell@stanford.edu>
 #
 # This file is part of the ClusterShell library.
 #
@@ -41,6 +41,7 @@ and stderr, or even more...)
 """
 
 import errno
+import logging
 import os
 import Queue
 import thread
@@ -51,6 +52,9 @@ from ClusterShell.Worker.fastsubprocess import Popen, PIPE, STDOUT, \
 from ClusterShell.Engine.Engine import EngineBaseTimer, E_READ, E_WRITE
 
 
+LOGGER = logging.getLogger(__name__)
+
+
 class EngineClientException(Exception):
     """Generic EngineClient exception."""
 
@@ -319,20 +323,27 @@ class EngineClient(EngineBaseTimer):
             try:
                 wcnt = os.write(wfile.fd, wfile.wbuf)
             except OSError, exc:
-                if (exc.errno == errno.EAGAIN):
+                if exc.errno == errno.EAGAIN:
+                    # _handle_write() is not only called by the engine but also
+                    # by _write(), so this is legit: we just try again later
                     self._set_writing(sname)
                     return
+                if exc.errno == errno.EPIPE:
+                    # broken pipe: log warning message and do NOT retry
+                    LOGGER.warning('%s: %s', self, exc)
+                    return
                 raise
             if wcnt > 0:
-                self.worker._on_written(self.key, wcnt, sname)
                 # dequeue written buffer
                 wfile.wbuf = wfile.wbuf[wcnt:]
                 # check for possible ending
                 if wfile.eof and not wfile.wbuf:
+                    self.worker._on_written(self.key, wcnt, sname)
                     # remove stream from engine (not directly)
                     self._engine.remove_stream(self, wfile)
                 else:
                     self._set_writing(sname)
+                    self.worker._on_written(self.key, wcnt, sname)
 
     def _exec_nonblock(self, commandlist, shell=False, env=None):
         """
@@ -478,12 +489,11 @@ class EnginePort(EngineClient):
                                                     id(self), fd_in, fd_out)
 
     def _start(self):
+        """Start port."""
         return self
 
     def _close(self, abort, timeout):
-        """
-        Close port pipes.
-        """
+        """Close port."""
         if not self._msgq.empty():
             # purge msgq
             try:
@@ -513,7 +523,15 @@ class EnginePort(EngineClient):
         """
         Port message send method that will wait for acknowledgement
         unless the send_once parameter if set.
+
+        May be called from another thread. Will generate ev_msg() on
+        Port event handler (in Port task/thread).
+
+        Return False if the message cannot be sent (eg. port closed).
         """
+        if self._msgq is None: # called after port closed?
+            return False
+
         pmsg = EnginePort._Msg(send_msg, not send_once)
         self._msgq.put(pmsg, block=True, timeout=None)
         try:
@@ -525,6 +543,8 @@ class EnginePort(EngineClient):
 
     def msg_send(self, send_msg):
         """
-        Port message send-once method (no acknowledgement).
+        Port message send-once method (no acknowledgement). See msg().
+
+        Return False if the message cannot be sent (eg. port closed).
         """
-        self.msg(send_msg, send_once=True)
+        return self.msg(send_msg, send_once=True)
